function statvec_disp(v,smooth)
%STATVEC_DISP  Displays statistics of vector fields.
%   STATVEC_DISP(V) displays statistics of a set of vector fields as a
%   function of the field number.
%
%   The first 3 subplots show the evolution of the velocity statistics,
%   for each velocity component:
%    - Mean velocity <Vx>, <Vy>, <Vz>  (where <> is the spatial average)
%    - rms velocity <Vx^2>^(1/2), <Vy^2>^(1/2), <Vz^2>^(1/2)
%    - standard deviation <(Vx-<Vx>)^2>^(1/2), <(Vy-<Vy>)^2>^(1/2), etc.
%
%   The last subplot shows the energy (summed over the two velocity
%   components):
%     E_total = (<Vx^2> + <Vy^2> + <Vz^2>) / 2
%     E_mean  = (<Vx>^2 + <Vy>^2 + <Vz>^2) / 2
%     E_turbulent = (<(Vx-<Vx>)^2> + <(Vy-<Vy>)^2 + <(Vz-<Vz>)^2>) / 2
%   (note that E_total = E_mean + E_turbulent)
%
%   STATVEC_DISP(V,SMOOTH) smoothes the statistics over SMOOTH consecutive
%   vector fields. Use SMOOTH=1 for no smoothing (by default).  Use
%   SMOOTH=0 for smoothing over the whole fields (display only one point).
%
%   Example:
%      v = loadvec('*.vc7');
%      statvec_disp(v,10);
%
%   See also STATF, HISTF, HISTVEC_DISP.


%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.10,  Date: 2013/03/14
%   This function is part of the PIVMat Toolbox


% History:
% 2004/04/22: v1.00, first version.
% 2005/02/23: v1.01, cosmetics
% 2005/09/06: v1.02, id.
% 2005/09/15: v1.03, output arg suppressed.
% 2006/04/27: v1.04, optimized; now reads the setname of v
% 2011/03/25: v1.05, loadset removed
% 2012/05/31: v1.06, help text improved
% 2013/03/14: v1.10, works with 3D fields

error(nargchk(1,2,nargin));

if (ischar(v) || iscellstr(v)), v=loadvec(v); end % new v1.04
maxframe=length(v);

if nargin<2, smooth=1; end
if smooth==0, smooth=maxframe; end

% average the statistics
sm_frame=1:floor(maxframe/smooth);
for num=sm_frame
    range=((num-1)*smooth+1):(num*smooth);
    sm_stat_vx(num)=statf(vec2scal(v(range),'ux'));
    sm_stat_vy(num)=statf(vec2scal(v(range),'uy'));
    if isfield(v(range),'vz')
        sm_stat_vz(num)=statf(vec2scal(v(range),'uz'));
    end
end

% displays the results
subplot(2,2,1);
if isfield(v(1),'vz')
    plot(sm_frame*smooth,[sm_stat_vx(sm_frame).mean],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).mean],'bs-',sm_frame*smooth,[sm_stat_vz(sm_frame).mean],'m^-');
    legend('<V_x>','<V_y>','<V_z>');
else
    plot(sm_frame*smooth,[sm_stat_vx(sm_frame).mean],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).mean],'bs-');
    legend('<V_x>','<V_y>');
end
xlabel('field num');
ylabel('m/s');

title('Mean velocity, <V_i>');
line([min(sm_frame*smooth) max(sm_frame*smooth)],[0 0])
ax=ylim;
ylim(max(abs(ax))*[-1 1]);

subplot(2,2,2);
if isfield(v(1),'vz')
    ke_tot=([sm_stat_vx(sm_frame).rms].^2 + [sm_stat_vy(sm_frame).rms].^2 + [sm_stat_vz(sm_frame).rms].^2)/2;
    ke_turb=([sm_stat_vx(sm_frame).std].^2 + [sm_stat_vy(sm_frame).std].^2 + [sm_stat_vz(sm_frame).std].^2)/2;
    ke_mean=([sm_stat_vx(sm_frame).mean].^2 + [sm_stat_vy(sm_frame).mean].^2 + [sm_stat_vz(sm_frame).mean].^2)/2;
else
    ke_tot=([sm_stat_vx(sm_frame).rms].^2 + [sm_stat_vy(sm_frame).rms].^2)/2;
    ke_turb=([sm_stat_vx(sm_frame).std].^2 + [sm_stat_vy(sm_frame).std].^2)/2;
    ke_mean=([sm_stat_vx(sm_frame).mean].^2 + [sm_stat_vy(sm_frame).mean].^2)/2;
end
plot(sm_frame*smooth,ke_tot,'ro-', sm_frame*smooth,ke_turb,'bs-', sm_frame*smooth,ke_mean,'m^-');
legend('E total','E turb','E mean');
xlabel('field num');
ylabel('E (m/s)^2');
title('Total, mean and turbulent kinetic energies');
ax=ylim;
ylim([0 max(ylim)]);

subplot(2,2,3);
if isfield(v(1),'vz')
    plot(sm_frame*smooth,[sm_stat_vx(sm_frame).std],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).std],'bs-', sm_frame*smooth, [sm_stat_vz(sm_frame).std],'m^-');
    legend('std(V_x)','std(V_y)','std(V_z)');
else
    plot(sm_frame*smooth,[sm_stat_vx(sm_frame).std],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).std],'bs-');
    legend('std(V_x)','std(V_y)');
end
xlabel('field num');
ylabel('m/s');
title('Standard deviation, std(V_i) = <(V_i-<V_i>)^2>^{1/2}');
ax=ylim;
ylim([0 max(ylim)]);

subplot(2,2,4);
if isfield(v(1),'vz')
    plot(sm_frame*smooth,[sm_stat_vx(sm_frame).rms],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).rms],'bs-',sm_frame*smooth, [sm_stat_vz(sm_frame).rms],'m^-');
    legend('rms(V_x)','rms(V_y)','rms(V_z)');
else
    plot(sm_frame*smooth,[sm_stat_vx(sm_frame).rms],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).rms],'bs-');
    legend('rms(V_x)','rms(V_y)');
end
xlabel('field num');
ylabel('m/s');
title('rms velocity, rms(V_i) = <V_i^2>^{1/2}');
ax=ylim;
ylim([0 max(ylim)]);

